#include "routedump.h"

#include <in_iface.h>
#include <c32comm.h>
#include <d32comm.h>

#include <mrouterclient.h>

CInfoDump::CInfoDump()
{
  rs.Connect(3);

}

void CInfoDump::ConstructL()
{
  iFs.Connect();
  iCC=CCnvCharacterSetConverter::NewL();
  iCC->PrepareToConvertToOrFromL(KCharacterSetIdentifierIso88591, iFs);
  state=CCnvCharacterSetConverter::KStateDefault;
}

CInfoDump::~CInfoDump()
{
  iFile.Close();
  iFs.Close();
  rs.Close();
}


void CInfoDump::FieldValue(const TDesC& name, const TDesC& value)
{
  CC().ConvertFromUnicode(iBuf, name);
  iFile.Write(iBuf);
  iFile.Write(_L8(" = "));
  CC().ConvertFromUnicode(iBuf, value);
  iFile.Write(iBuf);
  iFile.Write(_L8("\n"));
}

void CInfoDump::FieldValue(const TDesC& name, const TInetAddr& addr) {
  TBuf<100> nb;
  addr.Output(nb);
  FieldValue(name,nb);
}

void CInfoDump::FieldValue(const TDesC& name, const TInt i) {
  TBuf<100> nb;
  nb.Format(_L("%d"),i);
  FieldValue(name,nb);
}


void CInfoDump::CapsValue(const TDesC &name, const TInt val, const TCapsMaskEntry* skel) {
  CC().ConvertFromUnicode(iBuf, name);
  iFile.Write(iBuf);
  iFile.Write(_L8(" = "));
  if (!val) { 
    iFile.Write(_L8("<empty mask>"));
  } else {
    while(skel->iMask) {
      if (val & skel->iMask) {
        CC().ConvertFromUnicode(iBuf, TPtrC(skel->iName));
        iFile.Write(iBuf);
        iFile.Write(_L8(" "));
      }
      skel++;
    }
  }
  iFile.Write(_L8("\n"));
}

#define M(x) { x , _S( # x ) }

static const TCapsMaskEntry RATE_CAPS[] = {
  M(KCapsBps50),
  M(KCapsBps75),
  M(KCapsBps110),
  M(KCapsBps134),
  M(KCapsBps150),
  M(KCapsBps300),
  M(KCapsBps600),
  M(KCapsBps1200),
  M(KCapsBps1800),
  M(KCapsBps2000),
  M(KCapsBps2400),
  M(KCapsBps3600),
  M(KCapsBps4800),
  M(KCapsBps7200),
  M(KCapsBps9600),
  M(KCapsBps19200),
  M(KCapsBps57600),
  M(KCapsBps115200),
  M(KCapsBps230400),
  M(KCapsBps460800),
  M(KCapsBps1152000),
  M(KCapsBps4000000),
  M(KCapsBpsSpecial),
  { 0 }
};

void CInfoDump::DumpInterface(TSoInetInterfaceInfo &cif) {
  iFile.Write(_L8("INTERFACE\n\n"));
  FieldValue(_L("Name"),cif.iName);
  FieldValue(_L("Tag"),cif.iTag);
  iValue.Format(_L("%u"),cif.iState);
  FieldValue(_L("State"),iValue);
  iValue.Format(_L("%u"),cif.iSpeedMetric);
  FieldValue(_L("Speed"),iValue);
  iValue.Format(_L("%u"),cif.iFeatures);
  FieldValue(_L("Features"),iValue);
  FieldValue(_L("Address"),cif.iAddress);
  FieldValue(_L("Netmask"),cif.iNetMask);
  FieldValue(_L("Broadcast"),cif.iBrdAddr);
  FieldValue(_L("Gateway"),cif.iDefGate);
  FieldValue(_L("DNS1"),cif.iNameSer1);
  FieldValue(_L("DNS2"),cif.iNameSer2);
  iFile.Write(_L8("\n\n"));
}

void CInfoDump::DumpRoute(TSoInetRouteInfo &rif) {
  iFile.Write(_L8("ROUTE\n\n"));
  iValue.Format(_L("%u"),rif.iType);
  FieldValue(_L("Type"),iValue);
  iValue.Format(_L("%u"),rif.iState);
  FieldValue(_L("State"),iValue);
  iValue.Format(_L("%u"),rif.iMetric);
  FieldValue(_L("Metric"),iValue);
  FieldValue(_L("Destination"),rif.iDstAddr);
  FieldValue(_L("Netmask"),rif.iNetMask);
  FieldValue(_L("IFAddress"),rif.iIfAddr);
  FieldValue(_L("Gateway"),rif.iGateway);
  iFile.Write(_L8("\n\n"));
}



void CInfoDump::DumpInfotoFileL(const TDesC& FileName)
{
  User::LeaveIfError(iFile.Replace(Fs(), FileName, EFileWrite));
  CleanupClosePushL(iFile);

  RSocket socket;
  User::LeaveIfError(socket.Open(rs,KAfInet,KSockStream,KProtocolInetTcp));
  /*TInetAddr addr;
  addr.SetAddress(INET_ADDR(127,0,0,1));
  addr.SetPort(5843);
  socket.Bind(addr);*/
  TPckgBuf<TSoInetInterfaceInfo> iif;
  User::LeaveIfError(socket.SetOpt(KSoInetEnumInterfaces,KSolInetIfCtrl)); 
  TInt ret;
  while ((ret=socket.GetOpt(KSoInetNextInterface,KSolInetIfCtrl,iif))==KErrNone) {
    DumpInterface(iif());
  }

  TPckgBuf<TSoInetRouteInfo> irt;
  User::LeaveIfError(socket.SetOpt(KSoInetEnumRoutes,KSolInetRtCtrl));
  while ((ret=socket.GetOpt(KSoInetNextRoute,KSolInetRtCtrl,irt))==KErrNone) {
    DumpRoute(irt());
  }
  socket.Close();
  CleanupStack::PopAndDestroy(); //iFile
}

void CInfoDump::AddMRouterDefaultRouteL(const TDesC& FileName) {
  User::LeaveIfError(iFile.Replace(Fs(), FileName, EFileWrite));
  CleanupClosePushL(iFile);
  RSocket socket;
  User::LeaveIfError(socket.Open(rs,KAfInet,KSockStream,KProtocolInetTcp));
  iFile.Write(_L8("1"));

  //search for address and gateway
  TPckgBuf<TSoInetRouteInfo> irt;
  TPckgBuf<TSoInetRouteInfo> nrt;
  nrt().iIfAddr=TInetAddr(KInetAddrNone,0);
  nrt().iGateway=TInetAddr(KInetAddrNone,0);
  User::LeaveIfError(socket.SetOpt(KSoInetEnumRoutes,KSolInetRtCtrl));
  TInt ret;
  while ((ret=socket.GetOpt(KSoInetNextRoute,KSolInetRtCtrl,irt))==KErrNone) {
    if (irt().iNetMask.Address()==INET_ADDR(255,254,0,0)) {
      //this has the if address
      nrt().iIfAddr=irt().iIfAddr;
    }
  }
  iFile.Write(_L8("2"));
  if (nrt().iIfAddr.Address()==KInetAddrNone) {
    iFile.Write(_L8("3"));
    User::Leave(KErrNotFound);
    //    iEikonEnv->InfoMsg(_L("Route not found. Check that mRouter connection is up!")); 
    //return;
  }
  nrt().iDstAddr=TInetAddr(KInetAddrAny,KInetPortAny);
  nrt().iNetMask=TInetAddr(KInetAddrMaskAll,KInetPortAny);;
  nrt().iGateway=TInetAddr(INET_ADDR(169,254,1,68),0);
  nrt().iState=ERtReady;
  nrt().iType=ERtUser;
  nrt().iMetric=0;
  DumpRoute(nrt());
  iFile.Write(_L8("4"));
  TInt err=socket.SetOpt(KSoInetAddRoute,KSolInetRtCtrl,nrt);
  iValue.Format(_L("Err %d "),err);
  FieldValue(_L(""),iValue);
  iFile.Write(_L8("5"));
  socket.Close();
  iFile.Write(_L8("6"));
  //DumpInfotoFileL(FileName);
  CleanupStack::PopAndDestroy(); //iFile
}






void CInfoDump::DumpSerInfotoFileL(const TDesC& FileName)
{
  User::LeaveIfError(iFile.Replace(Fs(), FileName, EFileWrite));
  CleanupClosePushL(iFile);

  iFile.Write(_L8("1"));


  RCommServ cserv;
  User::LeaveIfError(cserv.Connect());
  iFile.Write(_L8("2"));
  User::LeaveIfError(cserv.LoadCommModule(_L("ecuart")));
  User::LeaveIfError(cserv.LoadCommModule(_L("btcomm")));
  User::LeaveIfError(cserv.LoadCommModule(_L("btdun")));
  User::LeaveIfError(cserv.LoadCommModule(_L("ircomm")));
  User::LeaveIfError(cserv.LoadCommModule(_L("ecmux")));
  iFile.Write(_L8("3"));
  TInt nump;
  User::LeaveIfError(cserv.NumPorts(nump));
  iFile.Write(_L8("4\n"));
  FieldValue(_L("Num Modules"),nump);
  for (int j=0;j<nump;j++) {
    TBuf<150> modn;
    TSerialInfo si;
    User::LeaveIfError(cserv.GetPortInfo(j,modn,si));
    iFile.Write(_L8("\n\nMODULE\n"));
    FieldValue(_L("NAME"),modn);
    iFile.Write(_L8("\n"));
    iFile.Write(_L8("5\n"));
    FieldValue(_L("PortName"),si.iName);
    FieldValue(_L("PortDesc"),si.iDescription);
    FieldValue(_L("LowUnit"),si.iLowUnit);
    FieldValue(_L("HighUnit"),si.iHighUnit);
  }

  RComm cpt;
  iFile.Write(_L8("a"));  
  TInt ret=cpt.Open(cserv,_L("COMM::4"),ECommPreemptable);
  if (ret!=KErrNone) {
    if (ret==KErrLocked) {
      iFile.Write(_L8("=locked."));
    } else {
      FieldValue(_L("Error"),ret);
    }
  } else { //no error -- get caps
    iFile.Write(_L8("b"));  
    TCommCaps2 caps;
    cpt.Caps(caps);  
    FieldValue(_L("Supp rate"),caps().iRate);
    CapsValue(_L("Supp rate"),caps().iRate,RATE_CAPS);
    FieldValue(_L("Supp databits"),caps().iDataBits);
    FieldValue(_L("Supp StopBits"),caps().iStopBits);
    FieldValue(_L("Supp Parity"),caps().iParity);
    FieldValue(_L("Supp Handshake"),caps().iHandshake);
    FieldValue(_L("Supp Signals"),caps().iSignals);
    FieldValue(_L("Supp Fifo"),caps().iFifo);
    FieldValue(_L("Supp SIR"),caps().iSIR);
  }

  //mrouter info
  /*  RmRouterClient mcl;
  mcl.ConnectL(TRUE);
  TmRConfig cfg;
  cfg.iBearer=EmR_USB;
  mcl.MRConfigL(&cfg);
  iFile.Write(_L8("MROUTER USB CONFIG\n"));
  FieldValue(_L("bearer"),cfg.iBearer);
  FieldValue(_L("bearerunit"),cfg.iBearerUnit);
  FieldValue(_L("bearerspeed"),cfg.iBearerSpeed);
  
  TUSBListenStatus ls;
  mcl.USBListenStatusL(&ls);
  
  FieldValue(_L("\nusb listen status"),ls);
  
  TBTListenStatus bls;
  mcl.BTListenStatusL(&bls);

  FieldValue(_L("\nbt listen status"),bls);

  */

  cserv.Close();
  CleanupStack::PopAndDestroy(); //iFile
}
